#include "CircularPathSmoothing.h"

namespace armarx::nav::algo
{
    // TODO move to Simox
    float
    getAngleBetweenVectors(const Eigen::Vector2f& v1, const Eigen::Vector2f& v2)
    {
        // Returns an angle between 0 and 180 degree (the minimum angle between the two vectors)
        const Eigen::Vector2f v1_vec_normed = v1.normalized();
        const Eigen::Vector2f v2_vec_normed = v2.normalized();
        const float dot_product_vec =
            v1_vec_normed(0) * v2_vec_normed(0) + v1_vec_normed(1) * v2_vec_normed(1);
        float yaw = acos(dot_product_vec);
        return yaw;
    }

    CircularPathSmoothing::Points
    CircularPathSmoothing::smooth(const Points& p)
    {

        Points points;

        points.push_back(p.front());

        // omit start and end
        for (size_t i = 1; i < (p.size() - 1); i++)
        {
            const Eigen::Vector2f& prev = p.at(i - 1);
            const Eigen::Vector2f& at = p.at(i);
            const Eigen::Vector2f& next = p.at(i + 1);

            // the largest radius for smoothing
            const float maxRadius = std::min((prev - at).norm(), (next - at).norm());

            const float radius = std::min(maxRadius, 300.F);

            const Eigen::Vector2f vIn = (prev - at).normalized();
            const Eigen::Vector2f vOut = (next - at).normalized();

            // const float phiIn  = ::math::Helpers::Angle(vIn);
            // const float phiOut = ::math::Helpers::Angle(vOut);

            // const float phi = phiIn + (phiOut - phiIn) / 2;

            // the angle between the lines
            const float phi = getAngleBetweenVectors(vIn, vOut);

            // const Eigen::Vector2f at_dir = Eigen::Rotation2Df(phi) * Eigen::Vector2f::UnitX();

            // const float rho = - sign(v_in.y()*v_out.x() - v_in.x() * v_out.y());
            // const float chi_0 = std::atan2(v_in.y(), v_in.x());
            // const float chi_end = std::atan2(v_out.y(), v_out.x());

            // const float l = radius * std::cos(phi / 2);
            const float l = radius / std::tan(phi / 2);

            points.push_back(at + Eigen::Vector2f(prev - at).normalized() * l);
            // points.push_back(at);
            points.push_back(at + Eigen::Vector2f(next - at).normalized() * l);

            // Eigen::Vector2f dirAtToPrev = (prev - at).normalized();
            // Eigen::Vector2f dirAtToNext = (next - at).normalized();

            // Eigen::Vector2f center =
            //     at + dirAtToPrev * l +
            //     radius * Eigen::Vector2f(dirAtToPrev.y(), -dirAtToPrev.x());

            // const float alpha_start =
            //     math::Helpers::Angle(Eigen::Vector2f(dirAtToPrev.y(), -dirAtToPrev.x()));
            // const float alpha_end =
            //     math::Helpers::Angle(Eigen::Vector2f(-dirAtToNext.y(), dirAtToNext.x()));

            // points.push_back(center + radius * Eigen::Vector2f(Eigen::Rotation2Df(alpha_end) * Eigen::Vector2f::UnitX()));
            // points.push_back(center + radius * Eigen::Vector2f(Eigen::Rotation2Df(alpha_start + (alpha_start - alpha_end)/2) * Eigen::Vector2f::UnitX()));
            // points.push_back(center + radius * Eigen::Vector2f(Eigen::Rotation2Df(alpha_start) * Eigen::Vector2f::UnitX()));
            // points.push_back(center);
        }

        points.push_back(p.back());

        return points;
    }
} // namespace armarx::nav::algo
