/**
 * This file is part of ArmarX.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @author     Fabian Reister ( fabian dot reister at kit dot edu )
 * @author     Christian R. G. Dreher ( c dot dreher at kit dot edu )
 * @date       2021
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#pragma once

// STD/STL
#include <functional>
#include <string>
#include <vector>

// Navigation
#include <Navigation/libraries/core/NavigatorInterface.h>
#include <Navigation/libraries/core/types.h>

namespace armarx::nav::client
{

    class Navigator
    {

    public:
        Navigator(core::NavigatorInterface& navigator);

        void moveTo(const core::Pose& pose, core::NavigationFrame frame);

        void moveTo(const std::vector<core::Pose>& waypoints, core::NavigationFrame frame);

        void moveTowards(const core::Direction& direction, core::NavigationFrame frame);

        void pause();

        void resume();

        void stop();

        void onGoalReached(const std::function<void(void)>& callback);

        void onGoalNearlyReached(const std::function<void(void)>& callback);

        void onWaypointReached(const std::function<void(int)>& callback);

        void onWaypointNearlyReached(const std::function<void(int)>& callback);

        enum class Status
        {
            Success,
            Failure
        };

        struct GoalReachedWaitResult
        {
            Status status;
        };

        GoalReachedWaitResult waitForGoalReached();

        GoalReachedWaitResult waitForGoalNearlyReached();

        struct WaypointReachedWaitResult
        {
            Status status;
            int waypointIndex;
        };

        WaypointReachedWaitResult waitForWaypointReached();

        WaypointReachedWaitResult waitForWaypointNearlyReached();

    protected:
    private:
        core::NavigatorInterface* navigator;
    };

} // namespace armarx::nav::client
