/**
 * This file is part of ArmarX.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @author     Fabian Reister ( fabian dot reister at kit dot edu )
 * @author     Christian R. G. Dreher ( c dot dreher at kit dot edu )
 * @date       2021
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#pragma once

#include <optional>

#include <Eigen/Geometry>

#include <SimoxUtility/meta/enum/EnumNames.hpp>
#include <VirtualRobot/VirtualRobot.h>

// #include <ArmarXCore/interface/serialization/Eigen/Eigen_fdi.h>

#include "Navigation/libraries/core/DynamicScene.h"
#include "Navigation/libraries/core/StaticScene.h"

namespace armarx::nav::core
{
    enum class NavigationFrame
    {
        Relative,
        Absolute
    };

    inline const simox::meta::EnumNames<NavigationFrame> NavigationFrameNames{
        {NavigationFrame::Relative, "Relative"},
        {NavigationFrame::Absolute, "Absolute"}};

    using Pose = Eigen::Isometry3f;
    using Pose2D = Eigen::Isometry2f;

    using Position = Eigen::Vector3f;
    using Direction = Eigen::Vector3f;

    using Rotation = Eigen::Matrix3f;

    using LinearVelocity = Eigen::Vector3f;
    using AngularVelocity = Eigen::Vector3f;

    struct VelocityLimits
    {
        float linear;
        float angular;
    };

    using Path = std::vector<Pose>;
    using Positions = std::vector<Position>;

    struct Twist
    {
        LinearVelocity linear;
        AngularVelocity angular;

        Pose poseDiff(float dt) const;

        static Twist Zero();
    };

    struct Waypoint
    {
        Pose pose;
    };

    struct Scene
    {
        std::optional<core::StaticScene> staticScene = std::nullopt;
        std::optional<core::DynamicScene> dynamicScene = std::nullopt;
        // TopologicMapPtr topologicMap;
        VirtualRobot::RobotPtr robot;
    };

} // namespace armarx::nav::core
