/**
 * This file is part of ArmarX.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @author     Fabian Reister ( fabian dot reister at kit dot edu )
 * @author     Christian R. G. Dreher ( c dot dreher at kit dot edu )
 * @date       2021
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#pragma once

#include <VirtualRobot/VirtualRobot.h>

#include "RobotAPI/components/ArViz/Client/Layer.h"
#include "RobotAPI/components/ArViz/Client/ScopedClient.h"

#include "IntrospectorInterface.h"
#include "Navigation/libraries/core/types.h"

// forward declaration
namespace armarx::nav::core
{
    struct Trajectory;
}

namespace armarx::nav::server
{

    class ArvizIntrospector : virtual public IntrospectorInterface
    {
    public:
        ArvizIntrospector(armarx::viz::Client arviz, const VirtualRobot::RobotPtr& robot);
        ~ArvizIntrospector() override = default;

        void onStackResult(const StackResult& result) override;

        // // Non-API
        ArvizIntrospector(ArvizIntrospector&& other) noexcept;
        ArvizIntrospector& operator=(ArvizIntrospector&&) noexcept;

    protected:
        void drawGlobalTrajectory(const core::Trajectory& trajectory);
        void drawLocalTrajectory(const core::Trajectory& trajectory);
        void drawRawVelocity(const core::Twist& twist);
        void drawSafeVelocity(const core::Twist& twist);

    private:
        viz::ScopedClient arviz;
        const VirtualRobot::RobotPtr robot;

        std::vector<viz::Layer> layers;
    };

} // namespace armarx::nav::server
